
/**
 * @file secmem.c
 *
 * Tento soubor dovoluje aplikacim dynamicky pracovat s knihovnou secmem.dll.
 * Knihovna secmem.dll dovoluje menit zabezpeceni oblasti pameti v privatnim
 * adresovem prostoru procesu.
 */

#include <windows.h>
#include "secmem.h"

/** Nazev knihovny */
#define DLL_LIBRARY_NAME               L"secmem.dll"
/** Nazev funkce pro nastaveni zabezpeceni oblasti pameti */
#define SECURE_FUNCTION_NAME           "SecureVirtualMemory"
/** Nazev funkce pro odstraneni zabezpeceni oblasti pameti. */
#define UNSECURE_FUNCTION_NAME         "UnsecureVirtualMemory"


/** Prototyp funkce pro nastaveni zabezpeceni oblasti pameti */
typedef HANDLE (WINAPI SECUREVIRTUALMEMORY)(PVOID StartAddress, ULONG_PTR Size, ULONG ProbeMode);
/** Prototyp funkce pro odstraneni zabezpeceni oblasti pameti. */
typedef BOOL (WINAPI UNSECUREVIRTUALMEMORY)(HANDLE SecureHandle);


/** Handle knihovny secmem.dll */
static HMODULE hLibrary = NULL;
/** Uchovava adresu funkce pro nastaveni zabezpeceni oblasti pameti. */
static SECUREVIRTUALMEMORY *_SecureVirtualMemory = NULL;
/** Uchovava adresu funkce pro odstraneni zabezpeceni oblasti pameti. */
static UNSECUREVIRTUALMEMORY *_UnsecureVirtualMemory = NULL;



/** Zabezpeci zadany region pameti v adresovem prostoru aktualniho procesu.
 *
 *  @param StartAddress Pocatecni adresa regionu.
 *  @param Size Velikost regionu v bajtech.
 *  @param ProbeMode Hodnota PAGE_XXX urcujici minimalni oporavneni, kterym
 *  zabezpeceny region musi v kazdem okamziku disponovat. Operacni sytem zablokuje
 *  vsechna volani NtProtectVirtualMemory pokousejici se o nastaveni omezenejsiho
 *  opravneni.
 *
 *  @return Funkce v pripade uspechu vrati handle zabezpeceneho regionu. V pripade
 *  neuspechu vrati hodnotu NULL.
 */
HANDLE SecMemSecureMemory(PVOID StartAddress, ULONG_PTR Size, ULONG ProbeMode)
{
  HANDLE Ret = NULL;

  if (_SecureVirtualMemory != NULL)
    Ret = _SecureVirtualMemory(StartAddress, Size, ProbeMode);

  return Ret;
}



/** Odstrani zabezpeceni ze zadaneho regionu pameti v adresovem prostoru aktualniho
 *  procesu.
 *
 *  @param SecureHandle Handle ciloveho zabezpeceneho regionu.
 *
 *  @return Funkce vraci hodnotu typu boolean indikujici uspech ci neuspech operace.
 */
BOOL SecMemUnsecureMemory(HANDLE SecureHandle)
{
  BOOL Ret = FALSE;

  if (_UnsecureVirtualMemory != NULL)
    Ret = _UnsecureVirtualMemory(SecureHandle);

  return Ret;
}



/** Nacte knihovnu secmem.dll do adresoveho prostoru procesu a najde v ni
 *  adresy funkci pro nastaveni a odstraneni zabezpeceni regionu pameti.
 *
 *  @return Vraci hodnotu boolean indikujici uspech ci neuspech operace.
 */
BOOL SecMemModuleInit(VOID)
{
  BOOL Ret = FALSE;

  hLibrary = LoadLibraryW(DLL_LIBRARY_NAME);
  Ret = hLibrary != NULL;
  if (Ret) {
    _SecureVirtualMemory = (SECUREVIRTUALMEMORY *)GetProcAddress(hLibrary, SECURE_FUNCTION_NAME);
    _UnsecureVirtualMemory = (UNSECUREVIRTUALMEMORY *)GetProcAddress(hLibrary, UNSECURE_FUNCTION_NAME);
    Ret = (_SecureVirtualMemory != NULL && _UnsecureVirtualMemory != NULL);
    if (!Ret) {
      FreeLibrary(hLibrary);
      hLibrary = NULL;
	}
  }

  return Ret;
}


/** Uvolni knihovnu secmem.dll z pameti procesu.
 */
VOID SecMemModuleFinit(VOID)
{
  _SecureVirtualMemory = NULL;
  _UnsecureVirtualMemory = NULL;
  FreeLibrary(hLibrary);
  hLibrary = NULL;

  return;
}
