
#include <windows.h>
#include "secmem.h"
#include "physmem.h"
#include "tests.h"



typedef enum _TEST_ACTION_TYPE {
  tatAllocPrivateMemory,
  tatMapPagingFileSection,
  tatMapFile,
  tatUserPhysicalPages,
  tatSecuredMemory,
  tatDevicePhysicalMemory
} TEST_ACTION_TYPE;


typedef struct _TEST_ACTION {
  LIST_ENTRY Entry;
  TEST_ACTION_TYPE ActionType;
  union {
    struct {
      PVOID BaseAddress;
	} PrivateMemory;

	struct {
      HANDLE SectonHandle;
      PVOID BaseAddress;
	} PagingFileSection;

	struct {
      HANDLE FileHandle;
      HANDLE SectionHandle;
      PVOID BaseAddress;
	} MappedFile;

	struct {
      PULONG_PTR PageNumbers;
      SIZE_T PageCount;
      PVOID BaseAddress;
      BOOLEAN Mapped;
	} Awe;

    struct {
      HANDLE SecureHandle;
	} SecuredRegion;

    struct {
      PVOID BaseAddress;
    } DevicePhysicalMemoryRegion;
  } u;
} TEST_ACTION, *PTEST_ACTION;



/************************************************************************/
/*                      PRIVATNI FUNKCE                                 */
/************************************************************************/



static VOID TestActionFree(PTEST_ACTION TestAction)
{
  switch (TestAction->ActionType) {
    case tatAllocPrivateMemory:
      if (TestAction->u.PrivateMemory.BaseAddress != NULL)
        VirtualFree(TestAction->u.PrivateMemory.BaseAddress, 0, MEM_RELEASE);
      break;

    case tatMapPagingFileSection:
      if (TestAction->u.PagingFileSection.BaseAddress != NULL)
        UnmapViewOfFile(TestAction->u.PagingFileSection.BaseAddress);

      if (TestAction->u.PagingFileSection.SectonHandle != NULL)
        CloseHandle(TestAction->u.PagingFileSection.SectonHandle);
      break;

    case tatMapFile:
      if (TestAction->u.MappedFile.BaseAddress != NULL)
        UnmapViewOfFile(TestAction->u.MappedFile.BaseAddress);

      if (TestAction->u.MappedFile.SectionHandle != NULL)
        CloseHandle(TestAction->u.MappedFile.SectionHandle);

      if (TestAction->u.MappedFile.FileHandle != NULL)
        CloseHandle(TestAction->u.MappedFile.FileHandle);
      break;

    case tatUserPhysicalPages:
      if (TestAction->u.Awe.BaseAddress != NULL) {
        MapUserPhysicalPages(TestAction->u.Awe.BaseAddress, TestAction->u.Awe.PageCount, NULL);
        VirtualFree(TestAction->u.Awe.BaseAddress, 0, MEM_RELEASE);
      }

      if (TestAction->u.Awe.PageCount > 0) {
        FreeUserPhysicalPages(GetCurrentProcess(), &TestAction->u.Awe.PageCount, TestAction->u.Awe.PageNumbers);
        HeapFree(GetProcessHeap(), 0, TestAction->u.Awe.PageNumbers);
      }
      break;

    case tatSecuredMemory:
      if (TestAction->u.SecuredRegion.SecureHandle != NULL)
        SecMemUnsecureMemory(TestAction->u.SecuredRegion.SecureHandle);
      break;

    case tatDevicePhysicalMemory:
      if (TestAction->u.DevicePhysicalMemoryRegion.BaseAddress != NULL)
        PhysMemUnmapMemory(TestAction->u.DevicePhysicalMemoryRegion.BaseAddress);
      break;
  }

  HeapFree(GetProcessHeap(), 0, TestAction);

  return;
}


static PTEST_ACTION TestActionCreate(TEST_ACTION_TYPE TestActionType)
{
  PTEST_ACTION Ret = NULL;

  Ret = (PTEST_ACTION)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(TEST_ACTION));
  if (Ret != NULL) {
    Ret->Entry.Blink = &Ret->Entry;
    Ret->Entry.Flink = &Ret->Entry;
    Ret->ActionType =  TestActionType;
  }

  return Ret;
}


static VOID TestSetAddAction(PTEST_SET TestSet, PTEST_ACTION TestAction)
{
  EnterCriticalSection(&TestSet->CritSec);
  TestAction->Entry.Blink = TestSet->ListHead.Blink;
  TestAction->Entry.Flink = &TestSet->ListHead;
  TestSet->ListHead.Blink = &TestAction->Entry;
  LeaveCriticalSection(&TestSet->CritSec);

  return;
}



/************************************************************************/
/*                         VEREJNE FUNKCE                               */
/*                         (MNOZINY TESTU)                              */
/************************************************************************/



PTEST_SET TestSetCreate(VOID)
{
  PTEST_SET Ret = NULL;

  Ret = (PTEST_SET)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(TEST_SET));
  if (Ret != NULL) {
    InitializeCriticalSection(&Ret->CritSec);
    Ret->ListHead.Blink = &Ret->ListHead;
    Ret->ListHead.Flink = &Ret->ListHead;
  }

  return Ret;
}


VOID TestSetFree(PTEST_SET TestSet)
{
  PTEST_ACTION TestAction = NULL;
  PTEST_ACTION Tmp = NULL;

  Tmp = CONTAINING_RECORD(TestSet->ListHead.Blink, TEST_ACTION, Entry);
  while (&Tmp->Entry != &TestSet->ListHead) {
    TestAction = Tmp;
    Tmp = CONTAINING_RECORD(Tmp->Entry.Blink, TEST_ACTION, Entry);
    TestActionFree(TestAction);
  }

  HeapFree(GetProcessHeap(), 0, TestSet);

  return;
}



/************************************************************************/
/*                           VEREJNE FUNKCE                             */
/*                              (TESTY)                                 */
/************************************************************************/



BOOL CreatePrivateMemoryAllocationTestAction(PTEST_SET TestSet, PVOID BaseAddress)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatAllocPrivateMemory);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.PrivateMemory.BaseAddress = BaseAddress;
    TestSetAddAction(TestSet, TestAction);
  }

  return Ret;
}


BOOL CreatePagingFileSectionTestAction(PTEST_SET TestSet, HANDLE SectionHandle, PVOID BaseAddress)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatMapPagingFileSection);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.PagingFileSection.SectonHandle = SectionHandle;
    TestAction->u.PagingFileSection.BaseAddress = BaseAddress;
    TestSetAddAction(TestSet, TestAction);
  }

  return Ret;
}


BOOL CreateMappedFileTestAction(PTEST_SET TestSet, HANDLE FileHandle, HANDLE SectionHandle, PVOID BaseAddress)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatMapFile);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.MappedFile.FileHandle = FileHandle;
    TestAction->u.MappedFile.SectionHandle = SectionHandle;
    TestAction->u.MappedFile.BaseAddress = BaseAddress; 
    TestSetAddAction(TestSet, TestAction);
  }

  return Ret;
}


BOOL CreateAweTestAction(PTEST_SET TestSet, PULONG_PTR PageNumbers, ULONG_PTR PageCount, PVOID BaseAddress)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatUserPhysicalPages);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.Awe.BaseAddress = BaseAddress;
    TestAction->u.Awe.PageCount = PageCount;
    TestAction->u.Awe.PageNumbers = (PULONG_PTR)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(ULONG_PTR) * PageCount);
    Ret = TestAction->u.Awe.PageNumbers != NULL;
    if (Ret) {
      CopyMemory(PageNumbers, TestAction->u.Awe.PageNumbers, sizeof(ULONG_PTR) * PageCount);
      TestSetAddAction(TestSet, TestAction);
    } else TestActionFree(TestAction);
  }

  return Ret;
}


BOOL CreateSecuredRegionTestAction(PVOID TestSet, HANDLE SecureHandle)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatSecuredMemory);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.SecuredRegion.SecureHandle = SecureHandle;
    TestSetAddAction(TestSet, TestAction);
  }

  return Ret;
}


BOOL CreateDevicePhysicalMemoryTestAction(PTEST_SET TestSet, PVOID BaseAddress)
{
  BOOL Ret = FALSE;
  PTEST_ACTION TestAction = NULL;

  TestAction = TestActionCreate(tatDevicePhysicalMemory);
  Ret = TestAction != NULL;
  if (Ret) {
    TestAction->u.DevicePhysicalMemoryRegion.BaseAddress = BaseAddress;
    TestSetAddAction(TestSet, TestAction);
  }

  return Ret;
}